"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DoorsSystem = void 0;
const SystemStatus_1 = require("./SystemStatus");
const Door_1 = require("../misc/Door");
const events_1 = require("../events");
/**
 * Represents a system for doors that must be manually opened.
 *
 * See {@link DoorsSystemEvents} for events to listen to.
 */
class DoorsSystem extends SystemStatus_1.SystemStatus {
    constructor(ship, systemType, data) {
        super(ship, systemType, data);
        this.cooldowns || (this.cooldowns = new Map);
        this.doors || (this.doors = []);
        this.doors = this.doors.map((door, i) => typeof door === "boolean"
            ? new Door_1.Door(this, i, door)
            : door);
        this.lastUpdate = 0;
    }
    /* eslint-disable-next-line @typescript-eslint/no-unused-vars */
    Deserialize(reader, spawn) {
        const numCooldown = reader.upacked();
        for (let i = 0; i < numCooldown; i++) {
            const systemType = reader.uint8();
            const cooldown = reader.float();
            this.cooldowns.set(systemType, cooldown);
        }
        for (const door of this.doors) {
            door.Deserialize(reader, false);
            if (door.isOpen) {
                this._openDoor(door.doorId, undefined, undefined);
            }
            else {
                this._closeDoor(door.doorId, undefined, undefined);
            }
        }
    }
    /* eslint-disable-next-line @typescript-eslint/no-unused-vars */
    Serialize(writer, spawn) {
        writer.upacked(this.cooldowns.size);
        for (const [systemType, cooldown] of this.cooldowns) {
            writer.uint8(systemType);
            writer.float(cooldown);
        }
        for (const door of this.doors) {
            writer.bool(door.isOpen);
        }
    }
    async _openDoor(doorId, player, rpc) {
        const door = this.doors[doorId];
        if (!door)
            return;
        door.isOpen = true;
        this.dirty = true;
        const ev = await door.emit(new events_1.DoorsDoorOpenEvent(this.room, this, rpc, player, door));
        if (ev.reverted) {
            door.isOpen = false;
            return;
        }
        if (ev.alteredDoor !== door) {
            door.isOpen = false;
            ev.alteredDoor.isOpen = true;
        }
    }
    /**
     * Open a door by its ID.
     * @param doorId The ID of the door to opne.
     */
    async openDoor(doorId) {
        if (this.room.hostIsMe) {
            await this._openDoor(doorId, this.room.myPlayer, undefined);
        }
        else {
            await this._sendRepair(doorId);
        }
    }
    async _closeDoor(doorId, player, rpc) {
        const door = this.doors[doorId];
        if (!door)
            return;
        door.isOpen = false;
        this.dirty = true;
        const ev = await door.emit(new events_1.DoorsDoorCloseEvent(this.room, this, rpc, player, door));
        if (ev.reverted) {
            door.isOpen = true;
            return;
        }
        if (ev.alteredDoor !== door) {
            door.isOpen = true;
            ev.alteredDoor.isOpen = false;
        }
    }
    /**
     * Close a door by its ID.
     * @param doorId The ID of the door to close.
     */
    async closeDoor(doorId) {
        this._closeDoor(doorId, this.room.myPlayer, undefined);
    }
    async HandleRepair(player, amount, rpc) {
        const doorId = amount & 0x1f;
        await this._openDoor(doorId, player, rpc);
    }
    Detoriorate(delta) {
        this.lastUpdate += delta;
        for (const [systemType, prevTime] of this.cooldowns) {
            const newTime = prevTime - delta;
            if (newTime < 0) {
                this.cooldowns.delete(systemType);
                this.lastUpdate = 0;
                this.dirty = true;
                continue;
            }
            this.cooldowns.set(systemType, newTime);
            if (this.lastUpdate > 1) {
                this.dirty = true;
                this.lastUpdate = 0;
            }
        }
    }
}
exports.DoorsSystem = DoorsSystem;
//# sourceMappingURL=DoorsSystem.js.map